/*****************************************************************
 * Program: Monte Carlo Localization Applet
 * Author:	Gert Kootstra
 * 			Artificial Intelligence Institute
 * 			University of Groningen, The Netherlands
 * Email:	G.Kootstra@ai.rug.nl
 * 			http://www.ai.rug.nl/~gert
 * Web:		http://www.ai.rug.nl/~gert/applets/mclJRE/ 
 * 
 * Description:
 * simulation of the localization of a robot with monte carlo techniques.
 * MCL is based on particle filtering. The applet simulates a robot (in blue)
 * that navigates through an environment based on obstacle avoidance. The red
 * dots depict the position of the particles. Every particle is a hypothesis
 * of the location (and orientation) of the robot. The algorithm assumes that
 * the map of the environment is known.
 * Initially, all the particles are randomly distributed over the environment.
 * The MCL algorithm works in three steps:
 * - Motion model. Every particle is moved on the map based on the odometry
 * 	 information of the robot. Resulting in a movement simular to that of
 * 	 the robot. Some Gaussian noise on the translation and rotation is added.
 * - Sensor model. A probability for every particle is calculated. This probabilty
 *   depends on the similarity between the sensor reading of the robot and the
 *   sensor reading that would result if the robot was at the position of the particle.
 * - Resampling: Based on these probabilities, the set of particles is resampled.
 *   This results in multiple occurences of the 'good' particle and no occurence
 *   of the 'bad' particles, leading to a petter prediction of the location.
 *****************************************************************/
import java.awt.*;
import java.awt.event.*;
import java.sql.Time;
import java.text.NumberFormat;
import java.util.*;
import java.applet.Applet;
import java.lang.Math;
import java.awt.image.*;
import javax.imageio.*;
import java.io.*;

import javax.imageio.stream.*;

import com.sun.jmx.snmp.Timestamp;

import java.lang.reflect.*;
import java.net.*;

class StatusBar extends Panel implements ActionListener, ItemListener {
	Label lStatus, lLST;
	Button bStartExp;
	TextField tfLogFile;
	Vehicle vehicle; 
	Choice cLST, cNrHypList;
	
	StatusBar(Vehicle v) {
		vehicle = v;

		setLayout(new BorderLayout());
		
		Panel panel = new Panel();
		panel.setLayout(new GridLayout(1,4));

		//tfLogFile = new TextField();
		//panel.add(tfLogFile,0);
		
		//bStartExp = new Button("Experiment");
		//bStartExp.addActionListener(this);
		//panel.add(bStartExp,1);

		String status = new String();
		if(vehicle.experimentRuns) {
			status += "Experiment running. ";
			status += "NrHyp: " + vehicle.nrHypotheses + ", Run: " + vehicle.runNr + " of " + vehicle.nrRuns + ", cycle: " + vehicle.stats.cycleNr;
		}
		else
			status += "Experiment not running. ";
		lStatus = new Label(status);
		panel.add(lStatus, 0);

		panel.add(new Label(), 1);
		panel.add(new Label(), 2);
		panel.add(new Label(), 3);
		panel.add(new Label("Loc.sel. thr"),4);

		cLST = new Choice();
		cLST.add("T = 0.35");
		cLST.add("T = 0.4");
		cLST.add("T = 0.45");
		cLST.add("T = 0.5");
		cLST.add("T = 0.6");
		cLST.addItemListener(this);
		vehicle.setLST(0);
		panel.add(cLST, 5);

		//cNrHypList = new Choice();
		//cNrHypList.add("all carying capacity");
		//cNrHypList.add("normal all");
		//cNrHypList.add("100, 500, 1500, 2500");
		//cNrHypList.add("200, 1000, 2000");
		//cNrHypList.add("1500");
		//cNrHypList.add("1675");
		//cNrHypList.addItemListener(this);
		//vehicle.setNrHypList(0);
		//panel.add(cNrHypList,3);
				
		add(panel,BorderLayout.NORTH);
	}
	
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == bStartExp) {
			vehicle.startExperiment(tfLogFile.getText());
			bStartExp.setEnabled(false);
		}
	}
	public void itemStateChanged(ItemEvent e) {
		if(e.getSource() == cLST) {
			vehicle.setLST(cLST.getSelectedIndex());
		}
		if(e.getSource() == cNrHypList) {
			vehicle.setNrHypList(cNrHypList.getSelectedIndex());
		}
	}

	public void updateStatus() {
		String status = new String();
		if(vehicle.experimentRuns) {
			status += "Experiment running. ";
			status += "NrHyp: " + vehicle.hypotheses.nrHypotheses + ", Run: " + vehicle.runNr + " of " + vehicle.nrRuns + ", cycle: " + vehicle.stats.cycleNr;
			if(vehicle.hypotheses.nichingMethod == Vehicle.NICHING_LOSE)
				status += ", lsT: " + vehicle.localSelectionT + ", lsTE: " + vehicle.localSelectionTE;
		}
		else {
			status += "Experiment not running. ";
			status += "NrHyp: " + vehicle.hypotheses.nrHypotheses + ", av: " + vehicle.averageNrHyp;
		}
		lStatus.setText(status);
	}
}

class ControlPanel extends Panel implements ActionListener, AdjustmentListener, ItemListener {
	Vehicle vehicle;

	Label lNrParticles, lSpeedSD, lRSpeedSD, lSensorSD, lPRandomDist, lPreserveDiversity, lWorld;
	Button bStartPause, bRestart, bKidnap, bExperiment;
	Scrollbar sbNrParticles, sbSpeedSD, sbRSpeedSD, sbSensorSD, sbPRandomDist;
	Choice cPreserveDiversity, cWorld;
	
	ControlPanel(Vehicle v) {
		vehicle = v;
		
		setLayout(new BorderLayout());

		//******************************************
		// Control panel
		//******************************************
		Panel controlPanel = new Panel();
		controlPanel.setLayout(new GridLayout(3,6));

		lSpeedSD = new Label("Trans. s: " + vehicle.hypotheses.speedStDev + " px");
		controlPanel.add(lSpeedSD,0);

		sbSpeedSD = new Scrollbar(Scrollbar.HORIZONTAL, (int)Math.rint(vehicle.hypotheses.speedStDev*1), 1, 0, 101);
		sbSpeedSD.addAdjustmentListener(this);
		controlPanel.add(sbSpeedSD,1);

		
		lRSpeedSD = new Label("Rot. s: " + vehicle.hypotheses.rspeedStDev + " rad");
		controlPanel.add(lRSpeedSD,2);

		sbRSpeedSD = new Scrollbar(Scrollbar.HORIZONTAL, (int)Math.rint(vehicle.hypotheses.rspeedStDev*100), 1, 0, 201);
		sbRSpeedSD.addAdjustmentListener(this);
		controlPanel.add(sbRSpeedSD,3);

		
		lSensorSD = new Label("Sensor s: " + vehicle.hypotheses.hypothesis.smStDev + " px");
		controlPanel.add(lSensorSD,4);

		sbSensorSD = new Scrollbar(Scrollbar.HORIZONTAL, (int)Math.rint(vehicle.hypotheses.hypothesis.smStDev*1), 1, 0, 101);
		sbSensorSD.addAdjustmentListener(this);
		controlPanel.add(sbSensorSD,5);
		
		
		lPRandomDist = new Label("P(rand dist): " + vehicle.hypotheses.pRandomDistributed);
		controlPanel.add(lPRandomDist,6);

		sbPRandomDist = new Scrollbar(Scrollbar.HORIZONTAL, (int)Math.rint(vehicle.hypotheses.pRandomDistributed*100), 1, 0, 101);
		sbPRandomDist.addAdjustmentListener(this);
		controlPanel.add(sbPRandomDist,7);

		
		lPreserveDiversity = new Label("Niching");
		controlPanel.add(lPreserveDiversity,8);

		cPreserveDiversity = new Choice();
		cPreserveDiversity.add("none");
		cPreserveDiversity.add("crowding");
		cPreserveDiversity.add("cl.o.t.worst");
		cPreserveDiversity.add("sharing");
		cPreserveDiversity.add("freq.dep.sel.20%");
		cPreserveDiversity.add("freq.dep.sel.1");
		cPreserveDiversity.add("local sel.");
		cPreserveDiversity.addItemListener(this);
		controlPanel.add(cPreserveDiversity, 9);

		
		lNrParticles = new Label("#particles: " + vehicle.hypotheses.nrHypotheses);
		controlPanel.add(lNrParticles,10);

		sbNrParticles = new Scrollbar(Scrollbar.HORIZONTAL, (int)Math.rint(vehicle.hypotheses.nrHypotheses/10), 10, 0, 1010);
		sbNrParticles.addAdjustmentListener(this);
		controlPanel.add(sbNrParticles,11);

		bKidnap = new Button("Kidnap");
		bKidnap.addActionListener(this);
		controlPanel.add(bKidnap,12);

//		bExperiment = new Button("Experiment");
//		bExperiment.addActionListener(this);
		controlPanel.add(new Label(),13);
		
		lWorld = new Label("World:");
		controlPanel.add(lWorld,14);
		
		cWorld = new Choice();
		cWorld.add("Ambiguous");
		cWorld.add("Non-ambiguous");
		cWorld.add("Complex");
		cWorld.addItemListener(this);
		controlPanel.add(cWorld,15);
		
		bStartPause = new Button("Start");
		bStartPause.addActionListener(this);
		controlPanel.add(bStartPause, 16);

		bRestart = new Button("Restart");
		bRestart.addActionListener(this);
		controlPanel.add(bRestart, 17);
		
		// en de rest
		
		add(controlPanel, BorderLayout.NORTH);

	}
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == bStartPause) {
			vehicle.startPause();
			if(vehicle.pause)
				bStartPause.setLabel("Start");
			else
				bStartPause.setLabel("Pause");
		}
		if(e.getSource() == bRestart) {
			vehicle.requestRestart();
			bStartPause.setLabel("Pause");
		}
		if(e.getSource() == bKidnap) {
			vehicle.kidnap();
		}
//		if(e.getSource() == bExperiment) {
//			vehicle.startExperiment();
//			bExperiment.setEnabled(false);
//		}
	}
	
	public void adjustmentValueChanged(AdjustmentEvent e) {
		if(e.getSource() == sbSpeedSD) {
			int value = sbSpeedSD.getValue(); // [0,100]
			vehicle.setSpeedSD((double)value);
			lSpeedSD.setText("Trans. s: " + vehicle.newspeedStDev + " px");
		}
		if(e.getSource() == sbRSpeedSD) {
			int value = sbRSpeedSD.getValue(); // [0,100]
			vehicle.setRSpeedSD((double)value/100); // [0,1]
			lRSpeedSD.setText("Rot. s: " + vehicle.newrspeedStDev + " rad");
		}
		if(e.getSource() == sbSensorSD) {
			int value = sbSensorSD.getValue(); // [0,100]
			vehicle.setSensorSD((double)value);
			lSensorSD.setText("Sensor s: " + vehicle.newsmStDev + " px");
		}
		if(e.getSource() == sbPRandomDist) {
			int value = sbPRandomDist.getValue(); // [0,100]
			vehicle.setPRandDist((double)value/100);
			lPRandomDist.setText("P(rand dist): " + vehicle.newpRandomDistributed);
		}
		if(e.getSource() == sbNrParticles) {
			vehicle.setNewNrHypotheses(sbNrParticles.getValue()*10);
			lNrParticles.setText("#particles: " + vehicle.newNrHypotheses);
		}
	}
	public void itemStateChanged(ItemEvent e) {
		if(e.getSource() == cPreserveDiversity) {
			vehicle.setNichingMethod(cPreserveDiversity.getSelectedIndex());
		}
		if(e.getSource() == cWorld) {
			vehicle.setWorld(cWorld.getSelectedIndex());
		}
	}
}

class Pixel {
	int x,y;
	Pixel(int a, int b) {
		x = a;
		y = b;
	}
}

class Log {
	String filename = new String();
	PrintStream fileOut;
	boolean fileExists = true;
	
	public void setFilename(String fname) {
		filename = fname;
	}
	
	public int openFile() {
		try {
			fileOut = new PrintStream(filename);
			return 1;
		}
		catch( FileNotFoundException e ) {
			System.out.println("File: " + filename + " could not be opened.");
			fileExists = false;
			return 0;
		}
	}
	
	public void println(String str) {
		if(fileExists)
			fileOut.println(str);
	}
}

class World extends Applet{
	int width, height;
	int imageWidth = 150, imageHeight = 150;
	BufferedImage world = new BufferedImage(imageWidth,imageHeight, BufferedImage.TYPE_INT_RGB);;
	Image scaledWorld;
	boolean firstTime = true;
	Vector objects = new Vector();
	boolean sonarBeam;
	char[] worldArray;
	String wFile;
	boolean ambiguous;
	
	World(boolean sB, String worldFile, boolean amb) {
		sonarBeam = sB;
		ambiguous = amb;
		URL url;
		try {
			url= new URL("http://www.csc.kth.se/~kootstra/applets/premature_convergence/" + worldFile);
			try {
				world = ImageIO.read(url);
			}
			catch(IOException e) {
				System.out.println("World file could not be opened");
			}
		}
		catch( MalformedURLException e) {
			System.out.println("Couldn't open URL");
		}
		
		//if(System.getProperty("os.name").compareTo(new String("Windows 2000"))==0 ||
		//		System.getProperty("os.name").compareTo(new String("Windows XP"))==0) {
		//	wFile = "./" + worldFile;
		//}
		//else
		//	wFile = "./" + worldFile;
		//wFile = "public_html/applets/premature_convergence/" + worldFile;
		//File f = new File(wFile);
		//try {
		//	world = ImageIO.read(f);
		//}
		//catch( IOException e ) {
		//	System.out.println("World file could not be opened" + worldFile);
		//}	
		
		getWorldSize();
		if(sonarBeam)
			getObjectVector();
		else
			getWorldArray();
	}
	
	void getWorldArray() {
		worldArray = new char[width*height];
		for(int y=0; y<height; y++) {
			for(int x=0; x<width; x++) {
				int rgb = world.getRGB(x,y);
				Color c = new Color(255,255,255);
				if(rgb != c.getRGB()) 
					worldArray[y*height + x] = 1;
				else
					worldArray[y*height + x] = 0;
			}
		}
	}
	
	void getObjectVector() {
		for(int y=0; y<height; y++) {
			for(int x=0; x<width; x++) {
				int rgb = world.getRGB(x,y);
				Color c = new Color(0,0,0);
				if(rgb == c.getRGB()) {
					Pixel p = new Pixel(x,y);
					objects.add(p);
				}
			}
		}
	}
	
	void getWorldSize() {
		width = world.getWidth();
		height = world.getHeight();
	}
	
	public void setSize(int w, int h) {
		width = w;
		height = h;
	}
	
	Image paint(double scale) {
		//g.drawImage(world.getScaledInstance(300,300,Image.SCALE_FAST),null,0,0);
		if(firstTime) {
			scaledWorld = world.getScaledInstance((int)Math.rint(imageWidth*scale),(int)Math.rint(imageWidth*scale),Image.SCALE_FAST); 
			firstTime = false;
		}			
		return scaledWorld;
	}
	
	void printObjects() {
		for(int i=0; i<objects.size(); i++) {
			Pixel p = (Pixel)objects.get(i);
			System.out.println(p.x + "," + p.y);
		}
	}
	
	void getObjects() {
		
	}
}

class Position {
	double x,y,a;
	
	static public void copyPosition(Position source, Position target) {
		target.x = source.x;
		target.y = source.y;
		target.a = source.a;
	}
	
	static public double normAngle(double a) {
		double a2 = a;
		while(a2 > Math.PI)
			a2 -= 2*Math.PI;
		while(a2 <= -Math.PI)
			a2 += 2*Math.PI;
		return a2;
	}
	public void nextPosition(double speed, double rspeed) {

		a = normAngle(a + rspeed);
		x += Math.cos(a)*speed;
		y += Math.sin(a)*speed;
	}
	
	public void setPosition(double X, double Y, double A) {
		x = X;
		y = Y;
		a = A;
	}

}

class Sonar {
	World world;
	int nrSonars = 16;
	double[] sonarAngle;
	double[] sonarReading;
	double beamWidth = Math.PI/16;
	double angleStep = 2*Math.PI/nrSonars;
	double maxReading = 20;
	boolean sonarBeam;
	Random R;
	
	Sonar(World w, boolean sB) {
		world = w;
		sonarBeam = sB;
		sonarAngle = new double[nrSonars];
		sonarReading = new double[nrSonars];
		R = new Random();
		
		double angle = -Math.PI/2; // first sonar left at 270 degrees
		for(int i=0; i<nrSonars; i++) {
			sonarAngle[i] = angle;
			angle = normAngle(angle + angleStep);
		}
	}
	
	public double getAngle(int i) {
		return sonarAngle[i];
	}
	
	public double getAngleMin(int i) {
		double sonarMin = normAngle(sonarAngle[i] - beamWidth/2);
		return sonarMin;
	}
	public double getAngleMax(int i) {
		double sonarMin = normAngle(sonarAngle[i] + beamWidth/2);
		return sonarMin;
	}
	
	public double normAngle(double a) {
		double a2 = a;
		while(a2 > Math.PI)
			a2 -= 2*Math.PI;
		while(a2 <= -Math.PI)
			a2 += 2*Math.PI;
		return a2;
	}

	public void calcReadings(Position pos) {
		if(sonarBeam) 
			calcReadingsBeam(pos);
		else
			calcReadingsNoBeam(pos);
	}

	public void addNoise(double sSD) {
		for(int i=0; i<nrSonars; i++) {
				sonarReading[i] += R.nextGaussian()*sSD;
		}
	}
	
	//sonar reading with raytracing, less computation
	public void calcReadingsNoBeam(Position pos) {
		for(int i=0; i<nrSonars; i++)
			sonarReading[i] = maxReading; // Double.MAX_VALUE;
		int x, y;
		for(int i=0; i<nrSonars; i++) {
			for(int d=0; d < maxReading; d++) {
				x = (int)(pos.x + Math.cos(sonarAngle[i] + pos.a)*d);
				y = (int)(pos.y + Math.sin(sonarAngle[i] + pos.a)*d);
				if(x>=0 && y>=0 && x<world.width && y<world.height) {
					if(world.worldArray[x + y*world.height] == 1) { // object
						sonarReading[i]=d;
						break;		// next sonar
					}
				}
				else {
					sonarReading[i] = d ; //maxReading;
					break;
				}
			}		

		}
	}
	
	//sonar reading based on a sonar beam, computationally heavy
	public void calcReadingsBeam(Position pos) {
		double min,max;
		for(int i=0; i<nrSonars; i++) {
			// init distance array
			sonarReading[i] = maxReading; // Double.MAX_VALUE;
		}
		
		for(int i=0; i<world.objects.size(); i++) {
			Pixel p = (Pixel)world.objects.get(i);
			// calculate angle absolute angle towards robot's center
			double dY = (double)p.y-pos.y;
			double dX = (double)p.x-pos.x;
			double d = Math.sqrt(dY*dY + dX*dX);
			if(d < maxReading) {
				double absAngle = Math.atan2(dY,dX);
				double relAngle = normAngle(absAngle - pos.a);
				for(int k=0 ; k<nrSonars; k++) {
					// Possibility for multiple sonars to sense the same object
					// More efficient when no overlap: only closest sensor needs to be considered
					min = getAngleMin(k);
					max = getAngleMax(k);
					if(max > min) {
						// Normal case
						if(relAngle >= min && relAngle <= max) {
							if(sonarReading[k] > d)
								sonarReading[k] = d;
							break; // no overlap in sonars
						}
					}
					else {
						// Sonar on boundary -Pi, Pi
						if(relAngle >= min || relAngle <= max) {
							if(sonarReading[k] > d)
								sonarReading[k] = d;
							break; // no overlap in sonars
						}
					}
				}
			}
		}
	}
	
	public double getReading(int i) {
		return sonarReading[i];
	}
}

class TestHypothesis {
	Sonar sonar;
	Position pos;
	double hY, hX, hA;
	double smStDev; 	// standard deviation of sensor model
	double maxGaussValue;
	
	TestHypothesis(World w, boolean sB, double smSD) {
		smStDev = smSD;
		sonar = new Sonar(w, sB);
		pos = new Position();
		maxGaussValue = Gaussian(0,0, smStDev);
	}
	public void setPosition(double x, double y, double a) {
		pos.setPosition(x,y,a);
	}
	public void setPosition(Position p) {
		pos = p;
	}
	
	public double getSonar(int i) {
		return sonar.getReading(i);
	}
	public void calcSonars() {
		sonar.calcReadings(pos);
	}
	
	public double Gaussian(double x, double m, double s) {
		return ((1/(s*Math.sqrt(2*Math.PI))) * Math.exp(-((x-m)*(x-m))/(2*s*s)) + 0.05);
	}
	
	public double getProbabilitySonar(int i, Sonar sonarRobot) {
		double readingR, readingH;
		readingR = sonarRobot.getReading(i);
		readingH = sonar.getReading(i);
		
		return (Gaussian(readingR, readingH, smStDev) / maxGaussValue);
	}
	
	public double getProbability(Sonar sonarRobot) {
		// return the probability of this hypothesis given the sonar readings of
		// the robot and the world
		calcSonars(); // Get sonar readings of this hypothesis

		double totalProbability = getProbabilitySonar(0, sonarRobot);
		for(int i=1; i<sonar.nrSonars; i++) {
			totalProbability *= getProbabilitySonar(i, sonarRobot);
		}
		return (totalProbability);
	}
	
	public void paint(Graphics2D g) {
		double x,y,x_old=0,y_old=0;
		double sum=0.0;
		g.setColor(Color.red);
		for(int i=50; i<250; i++) {
			x = i;
			y = Gaussian(x,150,2);
			y *= 100;
			if(i==50) {
				x_old = x;
				y_old = y;
			}
			g.drawLine((int)x,(int)y,(int)x_old,(int)y_old);
			x_old = x;
			y_old = y;
			if(i==150)
				g.setColor(Color.black);
			if(i%5==0)
				g.drawLine((int)x,0,(int)x,10);
		}

	}
}

class Hypothesis implements Comparable{
	Position pos;
	double probability=0.0;
	double energy;
	boolean selected = true;
	
	Hypothesis() {
		pos = new Position();
	}
	void copy(Hypothesis source) {
		Position.copyPosition(source.pos, pos);
		probability = source.probability;
		energy = source.energy;
		selected = false;
	}
	public void setPosition(double x, double y, double a) {
		pos.setPosition(x,y,a);
	}
	public void setEnergy(double e) {
		energy = e;
	}
	public void setProbability(double p) {
		probability = p;
	}
	public int compareTo(Object obj) {
		Hypothesis h2 = (Hypothesis)obj;
		return((int)Math.signum(probability - h2.probability));
	}
	public double normAngle(double a) {
		double a2 = a;
		while(a2 > Math.PI)
			a2 -= 2*Math.PI;
		while(a2 <= -Math.PI)
			a2 += 2*Math.PI;
		return a2;
	}
	double distanceTo(Hypothesis h) {
		// calc distance from particle to particle h. angle is in radials, multiplied by 16 to match x,y values
		return Math.sqrt((h.pos.x - pos.x)*(h.pos.x - pos.x) + (h.pos.y - pos.y)*(h.pos.y - pos.y) + 16*(normAngle(h.pos.a-pos.a))*(normAngle(h.pos.a-pos.a)));
	} 
}

class Bins {
	int nrBins;
	int[] bins;
	static int binSize = 2;
	static int nrAngles = 10;
	double angles = 2*Math.PI/nrAngles;
	int width, height;
	World world;
	
	Bins(World w) {
		world = w;
		width = world.width/binSize;
		height = world.height/binSize;
		nrBins = width * height * nrAngles;
		bins = new int[nrBins];
	}
	
	void fillBin(Position pos) {
		if(pos.x >= 0 && pos.x < world.width && pos.y >= 0 && pos.y < world.height)
			bins[(int)Math.floor((Math.PI+pos.a)/angles)*width*height + (int)Math.floor(pos.y/binSize)*width + (int)Math.floor(pos.x/binSize)]++;
	}
	int getBinSize(Position pos) {
		if(pos.x >= 0 && pos.x < world.width && pos.y >= 0 && pos.y < world.height)
			return(bins[(int)Math.floor((Math.PI+pos.a)/angles)*width*height + (int)Math.floor(pos.y/binSize)*width + (int)Math.floor(pos.x/binSize)]);
		else
			return 1;
	}
}

class RobotEstimation {
	// TODO: Add angle dimension to kernel estimator
	static int KNEAREST_ESTIMATOR = 0;
	static int KERNEL_ESTIMATOR = 1;
	Position estimatedPos = new Position();
	int estimator;
	double wsScale = 1;
	int wsWidth;
	int wsHeight;
	int kernelR = 2;
	double[][] worldSegments;
	double[][] kernel;
	
	RobotEstimation(int est, int w, int h) {
		estimator = est;
		wsWidth = (int)Math.round(wsScale * w);
		wsHeight = (int)Math.round(wsScale * h);
		if(estimator==KERNEL_ESTIMATOR) {
			worldSegments = new double[wsWidth][wsHeight];
			kernel = new double[2*kernelR+1][2*kernelR+1];
			setKernel();
		}
	}
	
	void nextEstimation(Vector hypotheses, int nrHypotheses) {
		if(estimator == KNEAREST_ESTIMATOR)
			kNearestNeighborEstimator(hypotheses, nrHypotheses);
		else
			kernelEstimator(hypotheses, nrHypotheses);
	}
	
	void kNearestNeighborEstimator(Vector hypotheses, int nrHypotheses) {
		// Calculate the estimated position of the robot by finding the highest density of particles
		// Two possible implementations:
		// - kth nearest neighbor estimation. Point with closest kth neighbor
		// - kernel estimation. Each particle places a gaussian kernel in the world. Highest pos is estimate
		// TODO: Check whether the position of the robot and the new hypothese set match in time. 
		//		It seems like the estimation is a little behind
		// TODO: How can we do this faster? Use blocks and check only particles in the block and neigboring blocks
		// Implementation of the kth nearest neighbor estimation
		int K = 5;
		double closestKvalue = Double.MAX_VALUE;
		int closestKhyp = nrHypotheses;
		for(int i=0; i<nrHypotheses; i++) {
			double[] distanceToTop = new double[K];
			for(int k=0; k<K; k++)
				distanceToTop[k] = Double.MAX_VALUE;
			double d;
			// Build the top-K list for particle i
			for(int j=0; j<nrHypotheses; j++) {
				d = ((Hypothesis)hypotheses.get(i)).distanceTo( ((Hypothesis)hypotheses.get(j)) );
				int k = K-1;
				while(k>=0 && d < distanceToTop[k]) {
					k = k-1;
				}
				if(k<K-1)
					distanceToTop[k+1] = d; // +1 since the while-loop decreases k one to many
			}
			// Closer kth neighbor?
			if(distanceToTop[K-1] < closestKvalue) {
				closestKvalue = distanceToTop[K-1];
				closestKhyp = i;
			}
		}		
		Position.copyPosition(((Hypothesis)hypotheses.get(closestKhyp)).pos, estimatedPos);
	}
	
	void kernelEstimator(Vector hypotheses, int nrHypotheses) {
		clearWorldSegments();
		for(int i=0; i<nrHypotheses; i++) {
				addKernel(((Hypothesis)hypotheses.get(i)).pos);
		}
		getPeakSegment();
	}
	void getPeakSegment() {
		double highestValue = 0;
		int highestX = -1;
		int highestY = -1;
		for(int i=0; i<wsWidth; i++) {
			for(int j=0; j<wsHeight; j++) {
				if(worldSegments[i][j]>highestValue) {
					highestValue = worldSegments[i][j];
					highestX = i;
					highestY = j;
				}
			}
		}
		estimatedPos.x = highestX;
		estimatedPos.y = highestY;
	}
	void setKernel() {
		for(int i=0;i<kernelR*2+1;i++) {
			for(int j=0;j<kernelR*2+1;j++) {
				kernel[i][j]=1;
			}
		}
	}
	void clearWorldSegments() {
		for(int i=0; i<wsWidth; i++) {
			for(int j=0; j<wsHeight; j++) {
				worldSegments[i][j]=0;
			}
		}
	}
	void addKernel(Position pos) {
		int x = (int)Math.round(wsScale*pos.x);
		int y = (int)Math.round(wsScale*pos.y);
		for(int i=Math.max(x-kernelR,0); i<Math.min(x+kernelR+1,wsWidth); i++) {
			for(int j=Math.max(y-kernelR,0); j<Math.min(y+kernelR+1,wsHeight); j++) {
				worldSegments[i][j] += kernel[i-x+kernelR][j-y+kernelR];
			}
		}
	}
	double getX() {
		return estimatedPos.x/wsScale;
	}
	double getY() {
		return estimatedPos.y/wsScale;
	}
	double getA() {
		return estimatedPos.a;
	}
	
}

class Hypotheses {
	TestHypothesis hypothesis;
	int nrHypotheses;
	double speedStDev;
	double rspeedStDev;
	double pRandomDistributed;
	int nichingMethod;
	Vector hypotheses, newHypotheses, hypotheses1, hypotheses2;
	int currentHypotheses = 1;
	double[] cumulativeP;
	double[] cumulativeExp;
	World world;
	Random R;
	boolean firstTime = true;
	double distanceSampleProportion = 0.2;
	int RESAMPLE_SIR = 0;
	int RESAMPLE_SUS = 1;
	int resampleMethod=RESAMPLE_SUS;
	
	double crowdingGG = 0.2;
	double crowdingCF = 0.01;
	double crowdingPoorest = 0.333;
	double sharingR = 150; 
	double sharingProp = 0.2;
	double hetAdvProp = 0.2;
	double localSelectionT;
	double localSelectionEout;
	Bins bins;
	RobotEstimation robotEstimation;
	int estimator = RobotEstimation.KERNEL_ESTIMATOR;
	
	
	Hypotheses(World w, boolean sB, int nrH, double sStDev, double rsStDev, 
			double smStDev, double pRD, int nM, double lsT, double lsTE) {
		world = w;
		nrHypotheses = nrH;    // Mind: changes if nichingMethod == NICHING_LOSE
		speedStDev = sStDev;
		rspeedStDev = rsStDev;
		pRandomDistributed = pRD;
		nichingMethod = nM;
		
		hypothesis = new TestHypothesis(w, sB, smStDev);
		hypotheses1 = new Vector(nrHypotheses);
		hypotheses2 = new Vector(nrHypotheses);
		for(int i=0; i<nrHypotheses; i++)
			hypotheses1.add(i, new Hypothesis());
		for(int i=0; i<nrHypotheses; i++)
			hypotheses2.add(i, new Hypothesis());
		hypotheses = hypotheses1;
		newHypotheses = hypotheses2;

		R = new Random(); 
		cumulativeP = new double[nrHypotheses];
		cumulativeExp = new double[nrHypotheses+1];
		uniformRandomDistribution();
		localSelectionT = lsT;
		localSelectionEout = lsTE * lsT;
		if(nichingMethod==Vehicle.NICHING_LOSE) {
			initEnergy();
			bins = new Bins(world);
		}
		robotEstimation = new RobotEstimation(estimator, w.width, w.height);
	}
	
	void initEnergy() {
		// Changed this, so that the initial hypotheses get lsT instead of half of it.
		// It is important that the hypotheses get some time to 'find' the solutions
		for(int i=0; i<nrHypotheses; i++) {
			((Hypothesis)hypotheses.get(i)).energy = localSelectionT;
		}
	}
	
	void uniformRandomDistribution() {
		double x=0,y=0,a=0;
		for(int i=0; i<nrHypotheses; i++) {
			x = world.width * R.nextDouble(); // [0,width)
			y = world.height * R.nextDouble(); // [0,height)
			a = Math.PI - 2*Math.PI * R.nextDouble(); //(-PI,PI];
			((Hypothesis)hypotheses.get(i)).pos.setPosition(x,y,a);
		}
	}
	
	void uniformDistribution() {
		double x=0,y=0,a=0;
		int k=0;
		for(int i=0; i<nrHypotheses; i++) {
			x = (i*25 + 10) % world.width;
			y = (i*25 + 10) / world.width * 25 + 10;
			a = Math.PI - 2*Math.PI * R.nextDouble(); //(-PI,PI];
			((Hypothesis)hypotheses.get(i)).pos.setPosition(x,y,a);
		}
	}
	
	double getRandomGaussian(double m, double s) {
		return (R.nextGaussian()*s + m);
	}
	
	double distanceWeight(double d) {
//		return 1;
		return d*d;
	}
	
	double getDistanceWeight(int i, int nrSamples) {
		double sumD = 0.0;
		int j;
		for(int k=0; k<nrSamples; k++) {
			// Estimate the distance by sampling
			j = R.nextInt(nrHypotheses);
			sumD += distance(((Hypothesis)hypotheses.get(i)).pos, ((Hypothesis)hypotheses.get(j)).pos);
		}
		return(distanceWeight(sumD/nrSamples));
	}
	double sharingFunction(double d) {
		// Sharing function with the use of r
		if(d < sharingR)
			return (1 - d/sharingR);
		else
			return 0;
	}
	double sharingFunction2(double d) {
		// Alternative sharing function, without the use of r
		return 1/d;
	}

	double getSharingValue(int i, int nrSamples) {
		// Sharing: Original algorithm proposed by Goldberg and Richardson (1987)
		// uses a variable r (sharingFunction). We propose to do without this 
		// variable and use s(d) = 1/d instead (sharingFunction2).
		// The algorithm samples a proportion of the population to calculate the
		// distance and the sharing value. Probabilities are devided by the sum
		// of sharing functions. This promotes distant particles.
		// Computational complexity: N * p*N , quadratic in the number of particles
		double sumS = 0.0;
		int j;
		for(int k=0; k<nrSamples; k++) {
			// Estimate the distance by sampling
			j = R.nextInt(nrHypotheses);
			sumS += sharingFunction2(distance(((Hypothesis)hypotheses.get(i)).pos, ((Hypothesis)hypotheses.get(j)).pos));
		}
		return sumS;
	}
	
	void getProbabilities(Sonar sonar) {
		// Calculate the probabilities of the hypotheses
		// Based on sensor readings. 
		double sumP=0.0 , sum = 0.0;
		double p=0.0;
		if(nichingMethod==Vehicle.NICHING_LOSE)
			cumulativeP = new double[nrHypotheses]; // Resize, since local selection had adaptive sample size
		// Calculate the probability of all heypotheses
		// Include advantage for small groups for preservation of diversity 
		for(int i=0; i<nrHypotheses; i++) {
			hypothesis.setPosition(((Hypothesis)hypotheses.get(i)).pos);
			p = hypothesis.getProbability(sonar);

			///////////////////////////
			// Calculate new probabilities for Sharing and Heterozygote Advantage
			///////////////////////////
			if(nichingMethod == Vehicle.NICHING_FDS20)
				p *= getDistanceWeight(i,(int)Math.rint(nrHypotheses*hetAdvProp));
			if(nichingMethod == Vehicle.NICHING_FDS1)
				p *= getDistanceWeight(i,1);
			else if(nichingMethod == Vehicle.NICHING_SHAR)
				p /= getSharingValue(i,(int)Math.rint(nrHypotheses*sharingProp));
			///////////////////////////
			
			((Hypothesis)hypotheses.get(i)).setProbability(p);
			if(i==0)
				cumulativeP[i] = p;
			else
				cumulativeP[i] = cumulativeP[i-1] + p;
		}
		// NB cumulative array does NOT sum up to 1, normailzation in sampleNewHypotheses
		
	}
	
	void copyHypothesis(Hypothesis source, Hypothesis target) {
		// Copy source to target
		target.copy(source);
	}
	Hypothesis copyHypothesis(Hypothesis source) {
		Hypothesis target = new Hypothesis();
		target.copy(source);
		return(target);
	}
	void copyHypotheses(Hypothesis[] source, Hypothesis[] target) {
		for(int i=0; i<nrHypotheses; i++) {
			copyHypothesis(source[i],target[i]);
		}
	}
	void copyHypotheses(Vector source, Vector target) {
		for(int i=0; i<nrHypotheses; i++) {
			copyHypothesis((Hypothesis)source.get(i),(Hypothesis)target.get(i));
		}
	}
	
	double distance(Position pos1, Position pos2) {
		return(Math.sqrt( (pos1.x-pos2.x)*(pos1.x-pos2.x) + (pos1.y-pos2.y)*(pos1.y-pos2.y)));
	}
	
	void resampleUsingSIR() {
		// O(N) algorithm for Sample Importance Resampling (SIR) filter
		// taken from:  (effective 2*N iterations)
		// Carpenter, J., Clifford, P. and Hearnhead, P. (1997) An improved particle
		// filter for non-linear problems. Tech. rep., Department of Statistics,
		// University of Oxford.

		// Fill cumulative exponentional distributed variables
//		newHypotheses = new Vector();
		
		if(nichingMethod==Vehicle.NICHING_LOSE)
			cumulativeExp = new double[nrHypotheses]; // Resize, since local selection had adaptive sample size

		cumulativeExp[0] = -Math.log(R.nextDouble());
		for(int i=1; i<nrHypotheses+1; i++) {
			cumulativeExp[i] = cumulativeExp[i-1] -Math.log(R.nextDouble());
		}
		int i=0;
		int j=0;
		double maxCumExp = cumulativeExp[nrHypotheses];
		double maxCumP = cumulativeP[nrHypotheses-1];	// For normalization
//		***** The original algorithm, the above is extended with functionality to ****//
//		*****	increase the probability to pick a hypotheses from a small group    ****//
		while(i < nrHypotheses) {
			if(cumulativeP[j]*maxCumExp/maxCumP > cumulativeExp[i]) {
				
				((Hypothesis)hypotheses.get(j)).selected = true;
				copyHypothesis(((Hypothesis)hypotheses.get(j)),((Hypothesis)newHypotheses.get(i)));
				i++;
			}
			else {
				j++;
			}
		}
	}
	
	void resampleUsingSUS() {
		// O(N) algorithm for Stochastic Uniform Sampling filter
		// taken from:  
		// Carpenter, J., Clifford, P. and Hearnhead, P. (1997) An improved particle
		// filter for non-linear problems. Tech. rep., Department of Statistics,
		// University of Oxford.
		
		int nrSamples = nrHypotheses;
		double maxCumP = cumulativeP[nrHypotheses-1];
		double curCumP = maxCumP/nrSamples * R.nextDouble();
		double stepCumP = maxCumP/nrSamples;

		int hypI = 0;
		int hypJ = 0;
		int i=0;
		while(hypI < nrSamples) {
			if(cumulativeP[hypI]>=curCumP) {
				// select hypotheses
				((Hypothesis)hypotheses.get(hypI)).selected = true;
				copyHypothesis(((Hypothesis)hypotheses.get(hypI)),((Hypothesis)newHypotheses.get(hypJ)));
				curCumP += stepCumP;
				hypJ++;
			}
			else
				hypI++;
			i++;
		}
	}
	
	Vector sampleUsingSUS(int nrSamples) {
		Vector sampleIDs = new Vector();
		double maxCumP = cumulativeP[nrHypotheses-1];
		double curCumP = maxCumP/nrSamples * R.nextDouble();
		double stepCumP = maxCumP/nrSamples;

		int hypI = 0;
		while(hypI < nrHypotheses) {
			if(cumulativeP[hypI]>=curCumP) {
				// select hypotheses
				sampleIDs.add(new Integer(hypI));
				curCumP += stepCumP;
			}
			else
				hypI++;
		}
		
		return sampleIDs;
	}
	
	void resampleUsingCrowding() {
		// Crowding, as original proposed by De Jong (1975)
		// The algorithm first selects a proportion GG individuals from the population
		// using fitness proportional selection (SUS). For each individual a crowding
		// factor is randomly sampled from the population. The closest element
		// in the crowding factor is replaced by the new individual.
		// Computational complexity is (GG)*N * CF*N
		
		// Select a proportion of the population that reproduces, the Generation Gap
		// Copy entire population
		copyHypotheses(hypotheses, newHypotheses);

		// Select proportion of population for reproduction
		int nrReproduce = (int)Math.rint(nrHypotheses*crowdingGG);
		int cfSize = (int)Math.rint(nrHypotheses*crowdingCF);
		Vector reproductionSample = sampleUsingSUS(nrReproduce);
		double d;
		int hypI;
		int hypJ;
		for(int ggI=0; ggI<reproductionSample.size(); ggI++) {
			hypI = ((Integer)reproductionSample.get(ggI)).intValue();
			// Randomly select sample from population of proportion size 'crowing factor'
			// and finds closest hypotheses
			double nnValue = Double.MAX_VALUE;
			int nnI = -1;
			for(int cfI=0; cfI<cfSize; cfI++) {
				hypJ = R.nextInt(nrHypotheses);
				d = distance( ((Hypothesis)hypotheses.get(hypI)).pos, ((Hypothesis)hypotheses.get(hypJ)).pos );
				if(d < nnValue) {
					nnValue = d;
					nnI = hypJ;
				}
			}
			// Replace closest element
			copyHypothesis(((Hypothesis)hypotheses.get(hypI)),((Hypothesis)newHypotheses.get(nnI)));
		}
	}
	
	void resampleUsingClosestOfTheWorst() {
		// resampleUsingClosestOfTheWorst resamples using the closest of the worst
		// algorithm, described in (Sedbrook, Wright and Wright (1991), according to
		// Mahfoud (1995).
		// The algorithm first selects a proportion GG individuals from the population
		// using fitness proportional selection (SUS). For each individual a crowding
		// factor is randomly sampled from the bottom 33% of the population. This
		// adds selection pressure to the crowding algorithm. The closest element
		// in the crowding factor is replaced by the new individual.
		// Computational complexity is (GG)*N * CF*N + NlogN (for sorting)
		
		// Select a proportion of the population that reproduces, the Generation Gap
		// Copy entire population
		copyHypotheses(hypotheses, newHypotheses);

		// Select proportion of population for reproduction
		int nrReproduce = (int)Math.rint(nrHypotheses*crowdingGG);
		int cfSize = (int)Math.rint(nrHypotheses*crowdingCF);
		int poorestSize = (int)Math.rint(nrHypotheses*crowdingPoorest);
		Vector reproductionSample = sampleUsingSUS(nrReproduce);
		Vector replaceSet = new Vector();
		double d;
		int hypI;
		int hypJ;
		// Order newHypotheses with increasing fitness
		Collections.sort(newHypotheses);
		for(int ggI=0; ggI<reproductionSample.size(); ggI++) {
			hypI = ((Integer)reproductionSample.get(ggI)).intValue();
			// Find closest element in the 'crowding factor' proportion of the population
			double nnValue = Double.MAX_VALUE;
			int nnI = -1;
			for(int cfI=0; cfI<cfSize; cfI++) {
				hypJ = R.nextInt(poorestSize); // Draw sample from poorest half of the population
				d = distance( ((Hypothesis)hypotheses.get(hypI)).pos, ((Hypothesis)newHypotheses.get(hypJ)).pos );
				if(d < nnValue) {
					nnValue = d;
					nnI = hypJ;
				}
			}
			// Replace closest element
			replaceSet.add(new Integer(nnI));
		}
		// Replace elements
		for(int i=0; i<reproductionSample.size(); i++) {
			hypI = ((Integer)reproductionSample.get(i)).intValue();
			hypJ = ((Integer)replaceSet.get(i)).intValue();
			copyHypothesis(((Hypothesis)hypotheses.get(hypI)),((Hypothesis)newHypotheses.get(hypJ)));
		}
	}

	void sampleNewHypotheses() {
		// For niching methods none and heterozygote advantage
		if(		nichingMethod == Vehicle.NICHING_NONE || 
				nichingMethod == Vehicle.NICHING_SHAR || 
				nichingMethod == Vehicle.NICHING_FDS20 || 
				nichingMethod == Vehicle.NICHING_FDS1) {
			if(resampleMethod==RESAMPLE_SIR)
				resampleUsingSIR();
			else if(resampleMethod==RESAMPLE_SUS)
				resampleUsingSUS();
		}
		else if(nichingMethod == Vehicle.NICHING_CROW) {
			resampleUsingCrowding();
		}
		else if(nichingMethod == Vehicle.NICHING_CLWO) {
			resampleUsingClosestOfTheWorst();
		}
	}	
	/*
	 * Code for the local selection schem
	 */
	void applyLocalSelection() {
		// Reset bins
		for(int i=0; i<bins.nrBins; i++)
			bins.bins[i] = 0;
		// Fill bins
		for(int i=0; i<nrHypotheses; i++) 
			bins.fillBin(((Hypothesis)hypotheses.get(i)).pos);
		
		// Recalculate probabilities based on bin sizes and 
		// reproduce. Calculate energy and decide to reproduce, die or continue
		
		newHypotheses.removeAllElements(); // empty the next generation
		for(int i=0; i<nrHypotheses; i++) {
			((Hypothesis)hypotheses.get(i)).energy += 
				((Hypothesis)hypotheses.get(i)).probability/bins.getBinSize(((Hypothesis)hypotheses.get(i)).pos) - 
				localSelectionEout;
			if(((Hypothesis)hypotheses.get(i)).energy > localSelectionT) {
				// Reproduce, add the hypothesis twice to the next generation
//				System.out.println("Repr, energy: " + ((Hypothesis)hypotheses.get(i)).energy + ", p: " + ((Hypothesis)hypotheses.get(i)).probability);
				((Hypothesis)hypotheses.get(i)).energy /= 2;   // Half the energy for both copies
				newHypotheses.add(copyHypothesis(((Hypothesis)hypotheses.get(i))));
				newHypotheses.add(copyHypothesis(((Hypothesis)hypotheses.get(i))));
			}
			else if(((Hypothesis)hypotheses.get(i)).energy < 0) {
				// Die, do not copy the hypothesis to the next generation
			}
			else {
				// Just copy this hypothesis to the next generation
				newHypotheses.add(copyHypothesis(((Hypothesis)hypotheses.get(i))));
			}
		}
	}
	/*
	 * End code for local selection
	 */
	
	void applyMotionModel(double speed, double rspeed) {
		// Apply the motion model to the new set of hypotheses
		// with chance of pRandomDistributed to be randomly distributed (explore)
		double speedNoise = 0.0, rspeedNoise = 0.0;
		double x,y,a;
		for(int i=0; i<nrHypotheses; i++) {
			speedNoise = getRandomGaussian(0,speedStDev);
			rspeedNoise = getRandomGaussian(0,rspeedStDev);
			
			((Hypothesis)hypotheses.get(i)).pos.nextPosition(speed+speedNoise,rspeed+rspeedNoise);
			if(R.nextDouble() < pRandomDistributed) { // random distribution of particles
				x = world.width * R.nextDouble(); // [0,width)
				y = world.height * R.nextDouble(); // [0,height)
				a = Math.PI - 2*Math.PI * R.nextDouble(); //(-PI,PI];
				((Hypothesis)hypotheses.get(i)).pos.setPosition(x,y,a);
			}
				
		}		
	//	System.out.println("Apply motion model to new hypotheses");
	}
	
	void applySensorModel(Sonar sonar) {
		// apply the sonar model
		//		- get the probabilities for all hypotheses
		//		- resample the hypotheses
		getProbabilities(sonar);
	}
	
	public void nextCycle(double speed, double rspeed, Sonar sonar) {
		// First apply the sensor model to calculate the possibilities for the
		// hypotheses and resample them.
		// Second, apply the motion model based on the robot's motion to the
		// new set of hypotheses
		applyMotionModel(speed,rspeed);				// Apply motion model, move all according to odometry
		applySensorModel(sonar);					// Apply sensor model with current sonar reading
		if(nichingMethod==Vehicle.NICHING_LOSE)
			applyLocalSelection();
		else
			sampleNewHypotheses();						// Resample the hypotheses
//		copyHypotheses(newHypotheses,hypotheses); // set the new hypotheses.

		// Switch hypotheses buffers
		if(currentHypotheses==1) {
			currentHypotheses = 2;
			hypotheses = hypotheses2;
			newHypotheses = hypotheses1;
		}
		else {
			currentHypotheses = 1;
			hypotheses = hypotheses1;
			newHypotheses = hypotheses2;
		}
		nrHypotheses = hypotheses.size();

		robotEstimation.nextEstimation(hypotheses, nrHypotheses);
//		hypotheses = newHypotheses;
		firstTime = false;
	}
	
	
	void paint(Graphics2D g, double s) {
		double x,y;
		double a;
		//NumberFormat nf = NumberFormat.getNumberInstance();

//		g.setColor(Color.red);
		for(int i=0; i<nrHypotheses; i++) {
			g.setColor(Color.red);
//			g.setColor(Color.black);
			
			x = ((Hypothesis)hypotheses.get(i)).pos.x;
			y = ((Hypothesis)hypotheses.get(i)).pos.y;
			a = ((Hypothesis)hypotheses.get(i)).pos.a;
//			g.drawLine((int)Math.rint(s*x), (int)Math.rint(s*y),(int)Math.rint(s*x), (int)Math.rint(s*y));
			g.fillOval((int)Math.rint(s*x-1), (int)Math.rint(s*y-1),2,2);
			//			g.drawLine((int)Math.round(x), (int)Math.round(y), (int)Math.round(x + Math.cos(a)*10), (int)Math.round(y + Math.sin(a)*10));

			
			//nf.setMinimumFractionDigits(1);
			//nf.setMaximumFractionDigits(2);
			//g.drawString(nf.format(hypotheses[i].probability), (int)x,(int)y);
		}
		x = robotEstimation.getX();
		y = robotEstimation.getY();
		a = robotEstimation.getA();
		g.setColor(Color.yellow);
		g.fillOval((int)Math.rint(s*x-7), (int)Math.rint(s*y-7),14,14);
		
	}
}

class Robot {
	World world;
	Sonar sonar;
	public 
	Position pos;
	int vR=5;
	double leftVelocity=0.0, rightVelocity=0.0;
	double speed=0.0, rspeed=0.0;
	boolean sonarsRead = false;
	Random R = new Random();
	
	Robot (World w, boolean sB) {
		world = w;
		sonar = new Sonar(w, sB);
		pos = new Position();
		pos.setPosition(12,12,-0.5);
	}
	void kidnap() {
		Random R = new Random();
		int x,y;
		double a;
		x = R.nextInt(world.width);
		y = R.nextInt(world.height);
		a = 2*Math.PI*R.nextDouble();
		while(world.worldArray[x + world.width*y] == 1) {
			x = R.nextInt(world.width);
			y = R.nextInt(world.height);
			a = 2*Math.PI*R.nextDouble();
		}
		pos.setPosition(x,y,a);
	}
	public double normAngle(double a) {
		double a2 = a;
		while(a2 > Math.PI)
			a2 -= 2*Math.PI;
		while(a2 <= -Math.PI)
			a2 += 2*Math.PI;
		return a2;
	}
	
	public double getSonar(int i) {
		if(!sonarsRead) {
			// Get nieuw sonar values if needed
			calcSonars();
		}
		return sonar.getReading(i);
	}
	public double getSonar(int i, double sSD) {
		// Get sonar with added noise
		if(!sonarsRead) {
			// Get nieuw sonar values if needed
			calcSonars(sSD);
		}
		return sonar.getReading(i);
	}

	public Sonar getSonar() {
		if(!sonarsRead) {
			// Get nieuw sonar values if needed
			calcSonars();
		}
		return sonar;
	}
	public Sonar getSonar(double sSD) {
		// Get sonar with added noise
		if(!sonarsRead) {
			// Get nieuw sonar values if needed
			calcSonars(sSD);
		}
		return sonar;
	}

	public void setVelocity2(double left, double right) {
		leftVelocity = left;
		rightVelocity = right;
		speed = (leftVelocity + rightVelocity)/2;
		rspeed = (leftVelocity-rightVelocity)/(2*vR);
	}

	public void calcSonars() {
//		pos.x = pos.x_new;
//		pos.y = pos.y_new;
//		pos.a = pos.a_new;
		sonar.calcReadings(pos);
		sonarsRead = true;
	}

	public void calcSonars(double sSD) {
		// Calculate the sonar sensors and add noise
		sonar.calcReadings(pos);
		sonar.addNoise(sSD);
		
		sonarsRead = true;
	}
	
	public void nextCycle() {
		// Calculate the new postition of the robot without noise. The noise is
		// present in the odometry used to 
		// Calculate the new sonar readings
//		double s = speed + R.nextGaussian()*vSD;
//		double rs = rspeed + R.nextGaussian()*rvSD;
		pos.nextPosition(speed,rspeed);
		sonarsRead = false;
	}
	
	double getSpeed(double vSD) {
		// Give the speed information adding some gaussian noise
		return speed + R.nextGaussian()*vSD;
	}
	
	double getRSpeed(double rvSD) {
		// Give the speed information adding some gaussian noise
		return rspeed + R.nextGaussian()*rvSD;
	}
	
	public void paint(Graphics2D g, double s) {
		//draw Sonars
		g.setColor(Color.red);
//		for(int i=0;i<sonar.nrSonars;i++) {
//			g.drawLine((int)Math.rint(s*pos.x),(int)Math.rint(s*pos.y),(int)Math.rint(s*pos.x + s*Math.cos(sonar.getAngle(i)+pos.a)*sonar.getReading(i)),(int)Math.rint(s*pos.y + s*Math.sin(sonar.getAngle(i)+pos.a)*sonar.getReading(i)));
//		}
		
		double[] x = new double[4];
		double[] y = new double[4];
		double[] a = new double[4];
		x[0] = pos.x;
		y[0] = pos.y;
		x[1] = world.width - y[0];
		y[1] = x[0];
		x[2] = world.width - x[0];
		y[2] = world.height - y[0];
		x[3] = y[0];
		y[3] = world.height - x[0];
		a[0] = pos.a;
		a[1] = Position.normAngle(a[0] + Math.PI/2);
		a[2] = Position.normAngle(a[0] + Math.PI);
		a[3] = Position.normAngle(a[0] + 3*Math.PI/2);
		// Draw the robot's position
		int n=0;
		g.setColor(Color.red);
		for(int i=0;i<sonar.nrSonars;i++) {
			g.drawLine((int)Math.rint(s*x[n]),(int)Math.rint(s*y[n]),(int)Math.rint(s*x[n] + s*Math.cos(sonar.getAngle(i)+a[n])*sonar.getReading(i)),(int)Math.rint(s*y[n] + s*Math.sin(sonar.getAngle(i)+a[n])*sonar.getReading(i)));
		}
		g.setColor(Color.green);
		g.fillOval((int)Math.rint(s*(x[n]-(vR))),(int)Math.rint(s*(y[n]-(vR))),(int)Math.rint(s*vR*2),(int)Math.rint(s*vR*2));
		
		// Draw the four possible locations of the robot in the ambiguous world.
		if(world.ambiguous) {
			for(n=1; n<4; n++) {
				g.setColor(Color.red);
				for(int i=0;i<sonar.nrSonars;i++) {
					g.drawLine((int)Math.rint(s*x[n]),(int)Math.rint(s*y[n]),(int)Math.rint(s*x[n] + s*Math.cos(sonar.getAngle(i)+a[n])*sonar.getReading(i)),(int)Math.rint(s*y[n] + s*Math.sin(sonar.getAngle(i)+a[n])*sonar.getReading(i)));
				}
				g.setColor(Color.blue);
				g.fillOval((int)Math.rint(s*(x[n]-(vR))),(int)Math.rint(s*(y[n]-(vR))),(int)Math.rint(s*vR*2),(int)Math.rint(s*vR*2));
			}
		}
		// draw Vehicle

////		g.setColor(new Color(100,100,100));
//		g.setColor(Color.blue);
//		g.fillOval((int)Math.rint(s*(pos.x-(vR))),(int)Math.rint(s*(pos.y-(vR))),(int)Math.rint(s*vR*2),(int)Math.rint(s*vR*2));
//		g.setColor(Color.green);
////		g.setColor(new Color(100,100,100));
//		g.drawLine((int)Math.rint(s*pos.x),(int)Math.rint(s*pos.y),(int)Math.rint(s*(pos.x + Math.cos(pos.a)*5*vR)),(int)Math.rint(s*(pos.y + Math.sin(pos.a)*5*vR)));
	}
}

class Stats {
	// TODO: Log type of world
	// TODO: Log distance from estimation to ro bot position (or symmteric version)
//	Hypotheses hypotheses;
	PrintStream logFile;
	File logF;
//	Robot robot;
	boolean fileExists = true;
	Vector knn = new Vector(4);
	boolean initKnnVector = true;
	int preConvCounter = 0;
	double dT = 10.0;
	double dT2 = 5;
	int maxPreConvCounter = 5;		// nr of cycles a cluster may appear empty before convergence
	int maxNrCycles = 500;		    // max number of cycles before experiment stops
	int logNr = 0;
	boolean converged = false;
	boolean restartRequest = false;
	boolean printTableHeading = true;
	int cycleNr = 0;
	double[] x = new double[4];
	double[] y = new double[4];
	double[] a = new double[4];
	int[] sizeCluster = new int[4];
	int[] sizeCluster2 = new int[4];
	int[] sizeClusterBig = new int[4];
	int timeToConvergence3=0;
	int timeToConvergence2=0;
	int timeToConvergence1=0;
	int preConvCounter4 = 0;
	int preConvCounter3 = 0;
	int preConvCounter2 = 0;
	int preConvCounter1 = 0;
	boolean cluster1conv = false;
	boolean cluster2conv = false;
	boolean cluster3conv = false;
	boolean cluster4conv = false;
	long beginTime;
	int nrClusters=0;
	Sonar sonar;
	World world;
	boolean contWhenConv;
	int logType;
	double localSelectionT;
	long totNrHyp = 0;
	long totNrHypConv3 = 0;
	long totNrHypConv2 = 0;
	long totNrHypConv1 = 0;

	
	Stats(String filename, Hypotheses hypotheses, World w, int[] nrHypList, int nrRuns, int nichingMethod, double lsT, boolean cWC, int lT) {
//		hypotheses = h;
//		robot = r;
		world = w;
		sonar = new Sonar(world, false);
		localSelectionT = lsT;
		contWhenConv = cWC;
		logType = lT;
		String filebase = new String();
		String fileext = new String();
		String filename2 = new String();
		filebase = filename.substring(0,filename.length()-4);
		fileext = filename.substring(filename.length()-4, filename.length());
		filename2 = filename;
		int versionNr=2;
		
		logF = new File(filename2);
		
		while(logF.exists()) {
			filename2 = filebase + "_" + versionNr + fileext;
			versionNr++;
			logF = new File(filename2);
		}
		
		try {
			logFile = new PrintStream(logF);
			System.out.println("File: " + logF.toString() + " opened for logging.");
		}
		catch( FileNotFoundException e ) {
			System.out.println("File: " + logF.toString() + " could not be opened for logging.");
			fileExists = false;
		}
		for(int i=0; i<4; i++) {
			knn.addElement(new Vector());
		}
		for(int n=0; n<4; n++) {
			sizeCluster[n] = 0;
		}			
		// Write heading
		String hypStr = new String();
		hypStr = "% TESTED_NR_OF_HYPOTHESES ";
		for(int i=0; i<nrHypList.length; i++) {
			hypStr += nrHypList[i] + " ";
		}
		System.out.println(hypStr + " lst " + localSelectionT);
		if(nichingMethod == Vehicle.NICHING_LOSE)
			logString("% NR_RUNS: " + nrRuns + ", MAX_NR_CYCLE: " + maxNrCycles + ", DIST_THRESHOLD: " + dT + ", DIST_THRESHOLD2: " + dT2 + ", MAXPRECONVCOUNTER: " + maxPreConvCounter + ", LOCALSELECTIONT: " + localSelectionT);
		else
			logString("% NR_RUNS: " + nrRuns + ", MAX_NR_CYCLE: " + maxNrCycles + ", DIST_THRESHOLD: " + dT + ", DIST_THRESHOLD2: " + dT2 + ", MAXPRECONVCOUNTER: " + maxPreConvCounter);

		logString(hypStr);
		String nichingString;
		if(nichingMethod == Vehicle.NICHING_NONE)
			nichingString = "NONE";
		else if(nichingMethod == Vehicle.NICHING_CROW)
			nichingString = "CROWDING";
		else if(nichingMethod == Vehicle.NICHING_CLWO)
			nichingString = "CLOSEST OF THE WORST";
		else if(nichingMethod == Vehicle.NICHING_SHAR)
			nichingString = "SHARING";
		else if(nichingMethod == Vehicle.NICHING_FDS20)
			nichingString = "FREQUENCY DEPENDENT SELECTION 20% SAMPLE";
		else if(nichingMethod == Vehicle.NICHING_FDS1)
			nichingString = "FREQUENCY DEPENDENT SELECTION 1 SAMPLE";
		else if(nichingMethod == Vehicle.NICHING_LOSE)
			nichingString = "LOCAL SELCTION";
		else
			nichingString = "UNKNOWN";
		logString("% NICHING_METHOD: " + nichingString);
		
		String worldString;
		if(world.ambiguous)
			worldString = "AMBIGUOUS";
		else
			worldString = "NON-AMBIGUOUS";
		logString("% WORLD: " + worldString);
		//logString("% cycleNr\tNrHyp1\tvarHyp1\tNrHyp2\tvarHyp2\tNrHyp3\tvarHyp3\tNrHyp4\tvarHyp4");

		if(hypotheses.robotEstimation.estimator == RobotEstimation.KERNEL_ESTIMATOR)
			logString("% ESTIMATOR: KERNEL ESTIMATION");
		else if(hypotheses.robotEstimation.estimator == RobotEstimation.KNEAREST_ESTIMATOR)
			logString("% ESTIMATOR: K-NEAREST NEIGHBOR ESTIMATION");

		beginTime = System.currentTimeMillis();
	}
	
	void restart(Vehicle v) {
		converged = false;
		restartRequest = false;
		preConvCounter = 0;
		timeToConvergence3=0;
		timeToConvergence2=0;
		timeToConvergence1=0;
		preConvCounter4 = 0;
		preConvCounter3 = 0;
		preConvCounter2 = 0;
		preConvCounter1 = 0;
		cluster1conv = false;
		cluster2conv = false;
		cluster3conv = false;
		cluster4conv = false;
		cycleNr = 0;
		totNrHyp = 0;
		totNrHypConv3 = 0;
		totNrHypConv2 = 0;
		totNrHypConv1 = 0;
		logNr++;
		for(int n=0; n<4; n++) {
			sizeCluster[n] = 0;
		}			
//		logFile.println("BEGIN NR_HYPOTHESES " + v.nrHypotheses + " RUN " + v.runNr + " NR_RUNS " + v.nrRuns);
//		logString("% cycleNr\tNrHyp1\tvarHyp1\tNrHyp2\tvarHyp2\tNrHyp3\tvarHyp3\tNrHyp4\tvarHyp4");
	}
	
	public double normAngle(double a) {
		double a2 = a;
		while(a2 > Math.PI)
			a2 -= 2*Math.PI;
		while(a2 <= -Math.PI)
			a2 += 2*Math.PI;
		return a2;
	}
	double distance(Hypothesis h, double x, double y, double a) {
		return Math.sqrt((h.pos.x - x)*(h.pos.x - x) + (h.pos.y - y)*(h.pos.y - y) + 16*(normAngle(h.pos.a-a))*(normAngle(h.pos.a-a)));
	} 
	
	int getNrClusters(Robot robot, Hypotheses hypotheses) {
		// Calculate the k-th nearest neighbor to the 4 possible positions
		// Calculate 4 possible locations
		int k = 3;
		double d;
		int nrCorrectClusters = 0;

		// Get the correct cluster positions
		x[0] = robot.pos.x;
		y[0] = robot.pos.y;
		a[0] = robot.pos.a;
		x[1] = robot.world.width - y[0];
		y[1] = x[0];
		a[1] = Position.normAngle(a[0] + Math.PI/2);
		x[2] = robot.world.width - x[0];
		y[2] = robot.world.height - y[0];
		a[2] = Position.normAngle(a[0] + Math.PI);
		x[3] = y[0];
		y[3] = robot.world.height - x[0];
		a[3] = Position.normAngle(a[0] + 3*Math.PI/2);

		// Set first k values in the knn vector
		if(initKnnVector) {
			// First time, vector initialisation
			for(int n=0; n<4; n++) {
				// Get first k values
				for(int i=0; i<k; i++) {
//					((Vector)knn.get(n)).add(i, new Double(distance(hypotheses.hypotheses[i],x[n],y[n],a[n])));
					((Vector)knn.get(n)).add(i, new Double(distance(((Hypothesis)hypotheses.hypotheses.get(i)),x[n],y[n],a[n])));
				}
				Collections.sort((Vector)knn.get(n));
			}
			initKnnVector = false;
		} else {
			for(int n=0; n<4; n++) {
				// Get first k values
				for(int i=0; i<k; i++) {
//					((Vector)knn.get(n)).set(i, new Double(distance(hypotheses.hypotheses[i],x[n],y[n],a[n])));
					((Vector)knn.get(n)).set(i, new Double(distance(((Hypothesis)hypotheses.hypotheses.get(i)),x[n],y[n],a[n])));
				}
				Collections.sort((Vector)knn.get(n));
			}
		}
		
		// Find the k-th nearest neighbors 
		for(int n=0; n<4; n++) {
			for(int i=k; i<hypotheses.nrHypotheses; i++) {
				d = distance(((Hypothesis)hypotheses.hypotheses.get(i)),x[n],y[n],a[n]);
				if(d < ((Double)((Vector)knn.get(n)).get(k-1)).doubleValue()) {
					((Vector)knn.get(n)).set(k-1, new Double(d));
					Collections.sort((Vector)knn.get(n));
				}
			}
		}			

		// Calculate the number of correct clusters
		for(int n=0; n<4; n++) {
			// Check if the k-th nearest neighbor is within the distance threshold
			if(((Double)((Vector)knn.get(n)).get(k-1)).doubleValue() < dT) {
				nrCorrectClusters++;
			}
		}
		
		return(nrCorrectClusters);
	}

	void calcClusterInfo(Robot robot, Hypotheses hypotheses, Vehicle v) {
		// This function is used to print statistics of the clusters:
		// - Number of hypotheses within threshold 1
		// - Number of hypotheses within threshold 2
		// - Mean Sum of Squared Errors from particles to closest cluster
		// TODO: Log the distance from the highest density in the population to the nearest 
		//		possible position (4 possibilities in symmetric worlds, 1 in non-symmetric.
		if(printTableHeading) {
			if(v.hypotheses.nichingMethod==Vehicle.NICHING_LOSE)
				logFile.println("BEGIN NR_HYPOTHESES " + v.nrHypotheses + " RUN " + v.runNr + " NR_RUNS " + v.nrRuns + " LST " + v.localSelectionT + " LSTE " + v.localSelectionTE);
			else
				logFile.println("BEGIN NR_HYPOTHESES " + v.nrHypotheses + " RUN " + v.runNr + " NR_RUNS " + v.nrRuns);

			if(logType==Vehicle.LOGTYPE_SIMPLE) {
				logString("% cycleNr nrHyp1 varHyp1 nrHyp2 varHyp2 nrHyp3 varHyp3 nrHyp4 varHyp4");
				logString("% NrHyp within dT: " + dT + " varHyp is variance of these hypotheses");
			}
			else if(logType==Vehicle.LOGTYPE_COMPLEX) {
				logString("% cycleNr nrPoints1 nrClPoints11 nrClPoints12 nrNNpoints2 nrClPoints2 nrClPoints22 nrNNpoints3 nrClPoints3 nrClPoints32 nrNNpoints4 nrClPoints4 nrClPoints42 MSSE estimationError");
				logString("% nrPoints, nr of hypotheses closest to this solution. closeF is the fraction of this nr that are within dT: " + dT + " and dT2: " + dT2);
			}
			printTableHeading = false;
		}
	
		// Get the total amount of particles
		totNrHyp += v.hypotheses.nrHypotheses;
		if(timeToConvergence3==0) // Not converged to 3 clusters
			totNrHypConv3 += v.hypotheses.nrHypotheses;
		if(timeToConvergence2==0)
			totNrHypConv2 += v.hypotheses.nrHypotheses;
		if(timeToConvergence1==0)
			totNrHypConv1 += v.hypotheses.nrHypotheses;

		// Calculate nr hypotheses per cluster
		nrClusters = 0;
		// Get the correct cluster positions
		x[0] = robot.pos.x;
		y[0] = robot.pos.y;
		a[0] = robot.pos.a;
		x[1] = robot.world.width - y[0];
		y[1] = x[0];
		a[1] = Position.normAngle(a[0] + Math.PI/2);
		x[2] = robot.world.width - x[0];
		y[2] = robot.world.height - y[0];
		a[2] = Position.normAngle(a[0] + Math.PI);
		x[3] = y[0];
		y[3] = robot.world.height - x[0];
		a[3] = Position.normAngle(a[0] + 3*Math.PI/2);
		double d;
		double[] varCluster = new double[4];
		for(int n=0; n<4; n++) {
			varCluster[n] = 0;
		}

		logFile.print(cycleNr + " ");
		for(int n=0; n<4; n++) {
			sizeCluster[n] = 0;
		}
		for(int n=0; n<4; n++) {
			sizeCluster2[n] = 0;
		}
		if(logType==Vehicle.LOGTYPE_COMPLEX) {
			for(int n=0; n<4; n++)
				sizeClusterBig[n] = 0;
		}
		// Calc stats
		if(logType==Vehicle.LOGTYPE_SIMPLE) {
			for(int i=0; i<hypotheses.nrHypotheses; i++) {
				for(int n=0; n<4; n++) {
					d = distance(((Hypothesis)hypotheses.hypotheses.get(i)),x[n],y[n],a[n]);
					if( d < dT) {
						sizeCluster[n]++;
						varCluster[n] += d*d;
					}
				}
			}	
			// Log
			for(int n=0; n<4; n++) 
				logFile.print(sizeCluster[n] + " " + varCluster[n]/sizeCluster[n] + " ");
			logFile.print("\n");
		} // if(logType==Vehicle.LOGTYPE_SIMPLE) {
		else if(logType==Vehicle.LOGTYPE_COMPLEX) {
			double SSE = 0;
			for(int i=0; i<hypotheses.nrHypotheses; i++) {
				double dMin = Double.MAX_VALUE;
				int closestCluster = 4;
				for(int n=0; n<4; n++) {
					d = distance(((Hypothesis)hypotheses.hypotheses.get(i)),x[n],y[n],a[n]);
					if( d < dT) {
						sizeCluster[n]++;
					}
					if( d < dT2) {
						sizeCluster2[n]++;
					}
					if( d < dMin) {
						closestCluster = n;
						dMin = d;
					}
				}
				sizeClusterBig[closestCluster]++;
				SSE += dMin*dMin;
			}
			// Log the nr of nearest points per cluster, size of cluster <dT, size of cluster <dT2, MSSE
			for(int n=0; n<4; n++) 
				logFile.print(sizeClusterBig[n] + " " + sizeCluster[n] + " " + sizeCluster2[n] + " ");
			logFile.print(SSE/hypotheses.nrHypotheses + " ");
			
			// Log the distance from the robot estimation to the robot (or nearest mirrored version)
			// TODO: Make this 3D
			double estX = hypotheses.robotEstimation.getX();
			double estY = hypotheses.robotEstimation.getY();
			if(world.ambiguous) {
				// Distance towards closest possible possition
				double estErrorLow = Double.MAX_VALUE;
				double estError;
				for(int n=0; n<4; n++) {
					estError = Math.sqrt( (estX - x[n])*(estX - x[n]) + (estY - y[n])*(estY - y[n]) );
					if(estError < estErrorLow) 
						estErrorLow = estError;
				}
				logFile.print(estErrorLow);
			}
			else {
				// Non ambiguous world. Distance to robot possition
				double estError = Math.sqrt( (estX - x[0])*(estX - x[0]) + (estY - y[0])*(estY - y[0]) );
				logFile.print(estError);
			}
			logFile.print("\n");
			
		}// else if(logType==Vehicle.LOGTYPE_COMPLEX) {
		
		if(!cluster1conv) {
			// check if cluster is converged
			if(sizeCluster[0]==0)
				preConvCounter1++;
			else
				preConvCounter1 = 0;
			if(preConvCounter1 >= maxPreConvCounter)
				cluster1conv = true;
			else
				nrClusters++;
		}
		if(!cluster2conv) {
			// check if cluster is converged
			if(sizeCluster[1]==0)
				preConvCounter2++;
			else
				preConvCounter2 = 0;
			if(preConvCounter2 >= maxPreConvCounter)
				cluster2conv = true;
			else
				nrClusters++;
		}
		if(!cluster3conv) {
			// check if cluster is converged
			if(sizeCluster[2]==0)
				preConvCounter3++;
			else
				preConvCounter3 = 0;
			if(preConvCounter3 >= maxPreConvCounter)
				cluster3conv = true;
			else
				nrClusters++;
		}
		if(!cluster4conv) {
			// check if cluster is converged
			if(sizeCluster[3]==0)
				preConvCounter4++;
			else
				preConvCounter4 = 0;
			if(preConvCounter4 >= maxPreConvCounter)
				cluster4conv = true;
			else
				nrClusters++;
		}
		if(timeToConvergence3==0) 
			if(nrClusters < 4)
				timeToConvergence3 = cycleNr;
		if(timeToConvergence2==0) 
			if(nrClusters < 3)
				timeToConvergence2 = cycleNr;
		if(timeToConvergence1==0) 
			if(nrClusters < 2)
				timeToConvergence1 = cycleNr;
		if(nrClusters < 2)
			converged = true;
		
	}

	void calcMissMatch(Robot robot, Hypotheses hypotheses) {
		x[0] = robot.pos.x;
		y[0] = robot.pos.y;
		a[0] = robot.pos.a;
		x[1] = robot.world.width - y[0];
		y[1] = x[0];
		a[1] = Position.normAngle(a[0] + Math.PI/2);
		x[2] = robot.world.width - x[0];
		y[2] = robot.world.height - y[0];
		a[2] = Position.normAngle(a[0] + Math.PI);
		x[3] = y[0];
		y[3] = robot.world.height - x[0];
		a[3] = Position.normAngle(a[0] + 3*Math.PI/2);

		Position pos = new Position();
		double[] s = new double[16];
		pos.setPosition(x[0],y[0],a[0]);
		sonar.calcReadings(pos);
		for(int i=0; i<16; i++) {
			s[i] = sonar.getReading(i);
		}
		
		boolean missmatch=false;
		double maxMiss = 0;
		for(int n=1; n<4; n++) {
			pos.setPosition(x[n],y[n],a[n]);
			sonar.calcReadings(pos);
			for(int i=0; i<16; i++) {
				if(sonar.getReading(i)!=s[i]) {
					if(Math.abs(sonar.getReading(i) - s[i]) >  maxMiss)
						maxMiss = Math.abs(sonar.getReading(i) - s[i]);
					missmatch = true;
				}
			}
		}
		if(missmatch) {
			System.out.println("miss: " + maxMiss);
			for(int n=0; n<4; n++) {
				pos.setPosition(x[n],y[n],a[n]);
				sonar.calcReadings(pos);
				logFile.print(n + "\t" + maxMiss + "\t");
				for(int i=0; i<16; i++) {
					logFile.print(sonar.getReading(i) + "\t");
				}
				logFile.print("\n");
			}
		}
	}
	
	void calcConvergence(Robot robot, Hypotheses hypotheses) {
		int nrClusters = getNrClusters(robot, hypotheses);
		if(nrClusters < 4) {
			preConvCounter++;
		} else {
			preConvCounter = 0;
		}
		if(preConvCounter >= maxPreConvCounter) {
			converged = true;
		}
		
	}
	void calcStats(Robot robot, Hypotheses hypotheses, Vehicle v) {
		// Function that is called each timestep to log statistics of the particle population
		if(fileExists) {
//			calcConvergence(robot,hypotheses);
			calcClusterInfo(robot, hypotheses, v); // Calculate cluster info and log it
			cycleNr++;
			if(!contWhenConv && converged) {
//				System.out.println("Converged, nrHyp:" + hypotheses.nrHypotheses + ", cycles: " + cycleNr);
				logFile.print("END_RUN CONVERGED TIMESTAMP " + Long.toString(System.currentTimeMillis()-beginTime) + " NR_HYPOTHESES " + hypotheses.nrHypotheses + " RUN "  + v.runNr + " ");
				logFile.print("TIME_TO_CONV3 " + timeToConvergence3 + " ");
				logFile.print("TIME_TO_CONV2 " + timeToConvergence2 + " ");
				logFile.print("TIME_TO_CONV1 " + timeToConvergence1 + " ");
				if(v.nichingMethod == Vehicle.NICHING_LOSE) {
					logFile.print("MEAN_NR_HYP " + totNrHyp/cycleNr + " ");
					if(timeToConvergence3!=0)
						logFile.print("MEAN_NR_HYP_CONV3 " + totNrHypConv3/timeToConvergence3 + " ");
					else
						logFile.print("MEAN_NR_HYP_CONV3 " + totNrHypConv3/cycleNr + " ");
					if(timeToConvergence2!=0)
						logFile.print("MEAN_NR_HYP_CONV2 " + totNrHypConv2/timeToConvergence2 + " ");
					else
						logFile.print("MEAN_NR_HYP_CONV2 " + totNrHypConv2/cycleNr + " ");
					if(timeToConvergence1!=0)
						logFile.print("MEAN_NR_HYP_CONV1 " + totNrHypConv1/timeToConvergence1 + "\n");
					else
						logFile.print("MEAN_NR_HYP_CONV1 " + totNrHypConv1/cycleNr + "\n");
				}
				else
					logFile.print("MEAN_NR_HYP " + totNrHyp/cycleNr + " ");
				restartRequest = true;
				printTableHeading = true;
			}
			else if(cycleNr >= maxNrCycles) {
//				System.out.println("Not converged");
				logFile.print("END_RUN NOT_CONVERGED TIMESTAMP " + Long.toString(System.currentTimeMillis()-beginTime) + " NR_HYPOTHESES " + hypotheses.nrHypotheses + " RUN " + v.runNr + " NR_CYCLES " + cycleNr + " ");
				logFile.print("TIME_TO_CONV3 " + timeToConvergence3 + " ");
				logFile.print("TIME_TO_CONV2 " + timeToConvergence2 + " ");
				logFile.print("TIME_TO_CONV1 " + timeToConvergence1 + " ");
				if(v.nichingMethod == Vehicle.NICHING_LOSE) {
					logFile.print("MEAN_NR_HYP " + totNrHyp/cycleNr + " ");
					// Mean nr hypotheses when 3,2,1 clusters were present
					if(timeToConvergence3!=0)
						logFile.print("MEAN_NR_HYP_CONV3 " + totNrHypConv3/timeToConvergence3 + " ");
					else
						logFile.print("MEAN_NR_HYP_CONV3 " + totNrHypConv3/cycleNr + " ");
					if(timeToConvergence2!=0)
						logFile.print("MEAN_NR_HYP_CONV2 " + totNrHypConv2/timeToConvergence2 + " ");
					else
						logFile.print("MEAN_NR_HYP_CONV2 " + totNrHypConv2/cycleNr + " ");
					if(timeToConvergence1!=0)
						logFile.print("MEAN_NR_HYP_CONV1 " + totNrHypConv1/timeToConvergence1 + "\n");
					else
						logFile.print("MEAN_NR_HYP_CONV1 " + totNrHypConv1/cycleNr + "\n");
				}
				else
					logFile.print("MEAN_NR_HYP " + totNrHyp/cycleNr + "\n");
				restartRequest = true;
				printTableHeading = true;
			}
		}
	}
	
	void logString(String str) {
		logFile.println(str);
	}
}

class Vehicle extends Applet implements Runnable{
	private int fieldWidth, fieldHeight;
	private Thread t;
	boolean pause = true;
	boolean resetBuf = false;
	BufferedImage bufferImage;
	Graphics2D bufferGraphics;
	double imageScale = 3.0;
	Image w;
	Robot robot;
	World world;
	Hypotheses hypotheses;
	Stats stats;
	Sonar sonar;

	double robotV_SD = 1;	// Noise on robot's translational velocity
	double robotRV_SD = 0.04;	// Noise on robot's rotational velocity +- 2 degrees
	double robotS_SD = 1;	// Noise on robot's sensor reading
	
	int nrHypotheses=1000,newNrHypotheses = nrHypotheses;
	double speedStDev = 2, newspeedStDev = speedStDev;
    double rspeedStDev = (double)Math.round(100*Math.PI/16)/100.0, newrspeedStDev = rspeedStDev;
	double smStDev = 5.0, newsmStDev = smStDev; 	// standard deviation of sensor model
	double pRandomDistributed = 0.0, newpRandomDistributed = pRandomDistributed;
	static int NICHING_NONE = 0;
	static int NICHING_CROW = 1;
	static int NICHING_CLWO = 2;
	static int NICHING_SHAR = 3;
	static int NICHING_FDS20 = 4;
	static int NICHING_FDS1 = 5;
	static int NICHING_LOSE = 6;
	int nichingMethod = NICHING_NONE, newNichingMethod = nichingMethod;
	String worldFile = new String("map2_bw.gif"), newworldFile = worldFile;
	boolean ambiguous = true;
	
	static final int LST_35 = 0;
	static final int LST_40 = 1;
	static final int LST_45 = 2;
	static final int LST_50 = 3;
	static final int LST_60 = 4;
	double localSelectionT = 0.35;
	double localSelectionTE = 0.2;
	double newLocalSelectionT = localSelectionT, newLocalSelectionTE = localSelectionTE;
	
	int counter=0;
	int paintCycle = 10;
	int waitingTime = 1;
	boolean sonarBeam = false;
	boolean restartRequest = false;
	boolean newValues = false;
	
	// Experiment variables
	int[] nrHypList = {100,200,500,1000,1500,2000,2500};
	int[] nrHypListLS = {232, 333, 499, 725, 959, 1277, 1675, 2236}; 
	double[] lsTList = {0.7, 0.65, 0.6, 0.55, 0.5, 0.45, 0.4, 0.35};
	
	int[] newNrHypList = nrHypList;
	int nrHypListID = 0;
	int nrRuns = 100; //100
	int runNr = 0;
	boolean experimentRuns = false;
	
	double averageNrHyp;
	int avNrHypLength = 25;
	int[] avNrHyp = new int[avNrHypLength];
	int avNrHypI = 0;
	
	static int LOGTYPE_SIMPLE = 0;
	static int LOGTYPE_COMPLEX = 1;
	int logType=LOGTYPE_COMPLEX;
	boolean contWhenConv = true;
	
	ControlPanel controlPanel;
	StatusBar statusBar;
					
	Vehicle(int w, int h) {
		fieldWidth = w;
		fieldHeight = h;
		world = new World(sonarBeam,worldFile, ambiguous);
		robot = new Robot(world, sonarBeam);
		hypotheses = new Hypotheses(world, sonarBeam, nrHypotheses, speedStDev, rspeedStDev,
				smStDev,pRandomDistributed, nichingMethod, localSelectionT, localSelectionTE);
	}
	
	void setFieldSize(int w, int h) {
		fieldWidth = w;
		fieldHeight = h;
	}
	
	void setControlPanel(ControlPanel cP) {
		controlPanel = cP;
	}
	void setStatusBar(StatusBar sB) {
		statusBar = sB;
	}
	
	public void start() {
		if (t==null) {
			t = new Thread( this, "Vehicle");
			t.start();
		}
	}
	
	public void run() {
		Thread thisThread = Thread.currentThread();
        while( t == thisThread ) {
        	//waitingTime = (2000-nrHypotheses)/20;
        	if(waitingTime < 1) waitingTime = 1;
        	//System.out.println(waitingTime);
        	waitingTime = 1;
    		try {
    			Thread.sleep( waitingTime );
			}
    		catch( InterruptedException e )
			{}
    		
    		if(newValues)
    			setNewValues();
    		if(restartRequest)
    			restart();
    		if(!pause) {
    			nextCycle();
    			repaint();
    			counter++;
    			//pause=true;
    		}
        }
	}

	synchronized public void nextCycle() {
		double[] prox = new double[robot.sonar.nrSonars];
		// Get sonar readings and control the robot's motion
		
		sonar = robot.getSonar(robotS_SD);
		for(int i=0 ; i<robot.sonar.nrSonars; i++)
			prox[i] = (1-(sonar.getReading(i)/robot.sonar.maxReading));	
		double right = 8 * (1 - (prox[0]*0.2 + prox[1]*0.3 + prox[2]*0.5 + prox[3]*0.7));
		double left  = 8 * (1 - (prox[8]*0.2 + prox[7]*0.3 + prox[6]*0.5 + prox[5]*0.7));
		robot.setVelocity2(left,right);
		robot.nextCycle(); // Calculate next cycle with noise

		// Estimate the robot's location with a particle filter.
		// The provided odometry information includes noise.
		// THe new sonar reading is calculated and noise added to it.
		hypotheses.nextCycle(robot.getSpeed(robotV_SD), robot.getRSpeed(robotRV_SD), robot.getSonar(robotS_SD));
		
		calcAverageNrHyp();
		
		if(experimentRuns)
			experimentControl();
		else
			statusBar.updateStatus();
	}
	void calcAverageNrHyp() {
		avNrHyp[avNrHypI % avNrHypLength] = hypotheses.nrHypotheses;
		avNrHypI++;
		if(avNrHypI % avNrHypLength == 0) {
			averageNrHyp = 0;
			for(int i=0; i<avNrHypLength; i++) 
				averageNrHyp += avNrHyp[i];
			averageNrHyp /= avNrHypLength;
		}
	}
	public void experimentControl() {
		// Calculate the statistics
		stats.calcStats(robot, hypotheses, this);
		if(stats.restartRequest) {
			// Start new experiment
			stats.restart(this);
			restartRequest = true;
			runNr++;
			if(runNr >= nrRuns) {
				// Enough data gathered. Increase nr of hypotheses and test again
				runNr = 0;
				nrHypListID++;
				if(nrHypListID < nrHypList.length) {
					newNrHypotheses = nrHypList[nrHypListID];
					if(nichingMethod==Vehicle.NICHING_LOSE)
						newLocalSelectionT = lsTList[nrHypListID];
				}
				else 
					stopExperiment();
			}
		}		
		statusBar.updateStatus();
	}
	public void startExperiment(String logFile) {
		nrHypList = newNrHypList;
		String logBase;
		if(System.getProperty("os.name").compareTo(new String("Windows 2000"))==0 ||
				System.getProperty("os.name").compareTo(new String("Windows XP"))==0) 
			logBase = new String("data/");
		else 
			logBase = new String("/staf/gert/software/Localization/data/");
		if(logFile.length()==0)
			logFile = "logFile.dat";
		stats = new Stats(logBase+logFile, hypotheses, world, nrHypList, nrRuns, nichingMethod, localSelectionT, contWhenConv, logType);
		stats.restart(this);
		
		runNr = 0;
		nrHypListID = 0;
		newNrHypotheses = nrHypList[nrHypListID];
		if(nichingMethod==Vehicle.NICHING_LOSE)
			newLocalSelectionT = lsTList[nrHypListID];
		restartRequest = true;
		experimentRuns = true;
	}
	public void stopExperiment() {
		pause = true;
		restartRequest = false;
		statusBar.bStartExp.setEnabled(true);
		experimentRuns = false;
		System.out.println("Experiment stopped");
	}
	public void paintBuffer(Graphics2D g) {
		if(!experimentRuns || counter%paintCycle == 0) {
			g.setColor(Color.white);
			g.fillRect(0,0,fieldWidth-1,fieldHeight-1);
			w = world.paint(imageScale);
			g.drawImage(w,null,this);
			robot.paint(g,imageScale);
			hypotheses.paint(g,imageScale);
		}
//		else {
//			g.setColor(Color.GRAY);
//			g.fillRect(250,250,100,100);
//			g.setColor(Color.WHITE);
//			String str = "cnt: " + counter;
//			g.drawString(str,275,300);
//		}
	}
	
	public void update(Graphics g) {
		paint(g);
	}
	
	public void resetBuffer() {
        // always keep track of the image size
        fieldWidth=getSize().width;
        fieldHeight=getSize().height;
        setFieldSize(fieldWidth,fieldHeight);
//        world.setSize(fieldWidth,fieldHeight);
        
        //    clean up the previous image
        if(bufferGraphics!=null){
            bufferGraphics.dispose();
            bufferGraphics=null;
        }
        if(bufferImage!=null){
            bufferImage.flush();
            bufferImage=null;
        }
        System.gc();

        //    create the new image with the size of the panel
        bufferImage=new BufferedImage(fieldWidth,fieldHeight, BufferedImage.TYPE_INT_RGB);
        bufferGraphics=bufferImage.createGraphics();
		bufferGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        bufferGraphics.setColor( Color.white );
        bufferGraphics.fillRect(0,0,fieldWidth,fieldHeight);

        resetBuf = false;
	}
	
	synchronized public void paint(Graphics graphics) {
		Graphics2D g = (Graphics2D)graphics;
        if(fieldWidth!=getSize().width || fieldHeight!=getSize().height || 
                bufferImage==null || bufferGraphics==null || resetBuf)
        	resetBuffer();
             
        if(bufferGraphics!=null){
        	paintBuffer(bufferGraphics);
        	g.drawImage(bufferImage,0,0,this);
        }	
	}
	void setNewValues() {
		hypotheses.speedStDev = newspeedStDev;
		hypotheses.rspeedStDev = newrspeedStDev ;
		hypotheses.hypothesis.smStDev = newsmStDev;
		hypotheses.pRandomDistributed = newpRandomDistributed;
		hypotheses.nichingMethod = newNichingMethod;
		speedStDev = newspeedStDev;
		rspeedStDev = newrspeedStDev ;
		smStDev = newsmStDev;
		pRandomDistributed = newpRandomDistributed;
		nichingMethod = newNichingMethod;
		localSelectionT = newLocalSelectionT;
		localSelectionTE = newLocalSelectionTE;
		newValues = false;
	}
	void setSpeedSD(double sSD) {
		newspeedStDev = sSD;
		newValues = true;
	}
	void setRSpeedSD(double rsSD) {
		newrspeedStDev = rsSD;
		newValues = true;
	}
	void setSensorSD(double mSD) {
		newsmStDev = mSD;
		newValues = true;
	}
	void setPRandDist(double p) {
		newpRandomDistributed = p;
		newValues = true;
	}
	void setNichingMethod(int nM) {
		newNichingMethod = nM;
		newValues = true;
	}
	void setNewNrHypotheses(int n) {
		newNrHypotheses = n;
	}
	void setWorld(int i) {
		if(i==0) {
			newworldFile = "map2_bw.gif";
			ambiguous = true;
		}
		if(i==1) {
			newworldFile = "map3_bw.gif";
			ambiguous = true;
		}
		if(i==2) {
			newworldFile = "map_bw.gif";
			ambiguous = false;
		}
		restartRequest = true;
		pause = true;
	}
	void setLST(int i) {
		switch(i) {
		case LST_35:
			newLocalSelectionT = 0.35; break;
		case LST_40:
			newLocalSelectionT = 0.4; break;
		case LST_45:
			newLocalSelectionT = 0.45; break;
		case LST_50:
			newLocalSelectionT = 0.5; break;
		case LST_60:
			newLocalSelectionT = 0.6; break;
		}
		newValues = true;
	}
	void setNrHypList(int i) {
		if(i==0) {
			newNrHypList = nrHypListLS;
		}
		else if(i==1) {
			newNrHypList = new int[7];
			newNrHypList[0] = 100;
			newNrHypList[1] = 200;
			newNrHypList[2] = 500;
			newNrHypList[3] = 1000;
			newNrHypList[4] = 1500;
			newNrHypList[5] = 2000;
			newNrHypList[6] = 2500;
		}
		else if(i==2) {
			newNrHypList = new int[4];
			newNrHypList[0] = 100;
			newNrHypList[1] = 500;
			newNrHypList[2] = 1500;
			newNrHypList[3] = 2500;
		}
		else if(i==3) {
			newNrHypList = new int[3];
			newNrHypList[0] = 200;
			newNrHypList[1] = 1000;
			newNrHypList[2] = 2000;
		}
		else if(i==4) {
			newNrHypList = new int[1];
			newNrHypList[0] = 1500;
		}
		else { //if(i==5) {
			newNrHypList = new int[1];
			newNrHypList[0] = 1675;
		}

	}
	void startPause() {
		pause = !pause;
	}
	
	void requestRestart() {
		restartRequest = true;
		pause = true;
	}
	void restart() {
		nrHypotheses = newNrHypotheses;
		localSelectionT = newLocalSelectionT;
		localSelectionTE = newLocalSelectionTE;
		worldFile = newworldFile;
		world = new World(sonarBeam,worldFile,ambiguous);
		robot = new Robot(world, sonarBeam);
		hypotheses = new Hypotheses(world, sonarBeam, nrHypotheses, speedStDev, rspeedStDev,
				smStDev,pRandomDistributed, nichingMethod, localSelectionT, localSelectionTE);
		restartRequest = false;
		pause = false;
	}
	void kidnap() {
		robot.kidnap();
	}
}

/********************************************
 * The Braitenberg class. This contains the
 * Controlbar and the Vehicle applet.
 */

public class Localization extends Applet{
    static int XExt = 600;
    static int YExt = 700;
	
    Vehicle vehicle;
    ControlPanel controlPanel;
    StatusBar statusBar;
    
    public void init() {
    	vehicle = new Vehicle(XExt, YExt);
    	controlPanel = new ControlPanel(vehicle);
    	statusBar = new StatusBar(vehicle);
    	
    	vehicle.setControlPanel(controlPanel);
    	vehicle.setStatusBar(statusBar);
    	
    	setLayout(new BorderLayout());
        Font font = new Font("SanSerif", Font.PLAIN, 10);
        setFont(font);
        add(controlPanel, BorderLayout.NORTH);

        vehicle.setFieldSize(XExt, YExt);
        vehicle.setVisible(true);
        add(vehicle, BorderLayout.CENTER );

        add(statusBar, BorderLayout.SOUTH);
        
        vehicle.init();
        vehicle.start();
    }

    public static void main( String args[] )
    {
    	Localization b = new Localization();
    	b.init();
    	Frame f = new Frame("Localization" );
    	f.setSize( XExt+15, YExt+30 );
    	f.setVisible(true);
    	//s.init();
    	f.add( "Center", b );
    	f.addWindowListener(
    			new WindowAdapter() {
    				public void windowClosing(WindowEvent e) {
    					System.exit(0);
    				}
    			}
    	);
    	f.show();
    	//              s.start();
    }
}
